<?php
/**
 * montonio/module-hire-purchase
 *
 * @category   montonio
 * @package    module-hire-purchase
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */
declare(strict_types=1);

namespace Montonio\Hirepurchase\Gateway\Payments\ResponseHandler\Handler;

use Magento\Sales\Api\Data\OrderInterface;
use Magento\Sales\Api\OrderManagementInterface;
use Magento\Sales\Api\OrderRepositoryInterface;
use Magento\Sales\Model\Order;
use Montonio\Hirepurchase\Exception\PaymentCancelledStatusException;
use Montonio\Hirepurchase\Gateway\Payments\ResponseHandler\ResponseHandlerInterface;

/**
 * Abandoned payment status response handler
 */
class AbandonedOrderStatusHandler implements ResponseHandlerInterface
{
    /**
     * @var OrderRepositoryInterface
     */
    private $orderRepository;

    /**
     * @var OrderManagementInterface
     */
    private $orderManagement;

    /**
     * @param OrderRepositoryInterface $orderRepository
     * @param OrderManagementInterface $orderManagement
     */
    public function __construct(
        OrderRepositoryInterface $orderRepository,
        OrderManagementInterface $orderManagement
    ) {
        $this->orderRepository = $orderRepository;
        $this->orderManagement = $orderManagement;
    }

    /**
     * @inheritdoc
     *
     * @throws PaymentCancelledStatusException
     */
    public function handle(OrderInterface $order, array $data): void
    {
        $message = __('Payment cancelled on payment gateway.');

        $storeValues = ['payment_method_name','amount','currency','merchant_reference','status'];
        $storeData = [];
        foreach ($storeValues as $v) {
            if (isset($data[$v])) {
                $storeData[$v] = $data[$v];
            }
        }
        if (!empty($storeData)) {
            $order->getPayment()->setAdditionalInformation('gateway_response', $storeData);
        }

        if ($order->getStatus() !== Order::STATE_CANCELED) {
            $order->addStatusHistoryComment($message);
            $this->orderRepository->save($order);
            $this->orderManagement->cancel($order->getId());
        }

        throw new PaymentCancelledStatusException($message);
    }
}
