<?php
/**
 * montonio/module-hire-purchase
 *
 * @category   montonio
 * @package    module-hire-purchase
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */
declare(strict_types=1);

namespace Montonio\Hirepurchase\Gateway\Payments\ResponseHandler\Handler;

use Magento\Sales\Api\Data\OrderInterface;
use Magento\Sales\Api\OrderRepositoryInterface;
use Montonio\Hirepurchase\Exception\PaymentUnknownStatusException;
use Montonio\Hirepurchase\Gateway\Payments\ResponseHandler\ResponseHandlerInterface;

/**
 * Unknown payment status response handler
 */
class UnknownOrderStatusHandler implements ResponseHandlerInterface
{
    /**
     * @var OrderRepositoryInterface
     */
    private $orderRepository;

    /**
     * @param OrderRepositoryInterface $orderRepository
     */
    public function __construct(
        OrderRepositoryInterface $orderRepository
    ) {
        $this->orderRepository = $orderRepository;
    }

    /**
     * @inheritdoc
     *
     * @throws PaymentUnknownStatusException
     */
    public function handle(OrderInterface $order, array $data): void
    {

        $storeValues = ['payment_method_name','amount','currency','merchant_reference','status'];
        $storeData = [];
        foreach ($storeValues as $v) {
            if (isset($data[$v])) {
                $storeData[$v] = $data[$v];
            }
        }
        if (!empty($storeData)) {
            $order->getPayment()->setAdditionalInformation('gateway_response', $storeData);
        }

        $message = __('Payment gateway returned %1 status for order #%2.', $data['status'], $order->getIncrementId());
        $order->addStatusHistoryComment($message);
        $this->orderRepository->save($order);

        throw new PaymentUnknownStatusException($message);
    }
}
