<?php
/**
 * montonio/module-hire-purchase
 *
 * @category   montonio
 * @package    module-hire-purchase
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */
namespace Montonio\Hirepurchase\Gateway\Split\Command;

/**
 * Class AspspsCommand for requesting available methods from gateway
 */
class AspspsCommand implements \Magento\Payment\Gateway\CommandInterface
{

    /**
     *
     * @param \Montonio\Hirepurchase\Gateway\Payments\Config\Config $config
     * @param \Magento\Framework\Stdlib\DateTime\TimezoneInterface $timezoneInterface
     * @param \Magento\Framework\UrlInterface $urlInterface
     * @param \Magento\Framework\HTTP\ClientFactory $httpClientFactory
     * @param \Magento\Payment\Gateway\Http\TransferBuilder $transferBuilder
     * @param \Psr\Log\LoggerInterface $logger
     */
    public function __construct(
        \Montonio\Hirepurchase\Gateway\Payments\Config\Config $config,
        \Magento\Framework\Stdlib\DateTime\TimezoneInterface $timezoneInterface,
        \Magento\Framework\UrlInterface $urlInterface,
        \Magento\Framework\HTTP\ClientFactory $httpClientFactory,
        \Magento\Payment\Gateway\Http\TransferBuilder $transferBuilder,
        \Psr\Log\LoggerInterface $logger
    ) {
        $this->config = $config;
        $this->timezoneInterface = $timezoneInterface;
        $this->urlInterface = $urlInterface;
        $this->httpClientFactory = $httpClientFactory;
        $this->transferBuilder = $transferBuilder;
        $this->logger = $logger;
    }

    /**
     * return request data
     *
     * @param array $commandSubject
     *
     * @return array|Command\ResultInterface
     */
    public function execute(array $commandSubject)
    {
        return $this->requestData();
    }

    protected function requestData()
    {
        $response = [
            'EE' => [
                '0' => [
                    'bic' => 'split_combined',
                    'name' => 'Pay later',
                    'logo_url' => 'https://public.montonio.com/logo/montonio_dark.png'
                ],
                '1' => [
                    'bic' => 'split_1',
                    'name' => 'Pay after 1 month',
                    'logo_url' => 'https://public.montonio.com/logo/montonio_dark.png'
                ],
                '2' => [
                    'bic' => 'split_2',
                    'name' => 'Pay after 2 months',
                    'logo_url' => 'https://public.montonio.com/logo/montonio_dark.png'
                ],
                '3' => [
                    'bic' => 'split_3',
                    'name' => 'Pay after 3 months',
                    'logo_url' => 'https://public.montonio.com/logo/montonio_dark.png'
                ]
            ]
        ];

        return $response;
    }
}
