<?php

declare(strict_types=1);

namespace Montonio\Hirepurchase\Model\Action;

use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Framework\Exception\InputException;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Sales\Api\OrderRepositoryInterface;

/**
 * Action class responsible for providing order object using increment id.
 */
class RetrieveOrderByIncrementId
{
    /**
     * @var OrderRepositoryInterface
     */
    private $orderRepository;

    /**
     * @var SearchCriteriaBuilder
     */
    private $searchCriteriaBuilder;

    /**
     * @param OrderRepositoryInterface $orderRepository
     * @param SearchCriteriaBuilder $searchCriteriaBuilder
     */
    public function __construct(
        OrderRepositoryInterface $orderRepository,
        SearchCriteriaBuilder $searchCriteriaBuilder
    ) {
        $this->orderRepository = $orderRepository;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
    }

    /**
     * Retrieve order by increment id.
     *
     * @param string $incrementId
     *
     * @return OrderInterface
     *
     * @throws InputException
     */
    public function execute(string $incrementId): OrderInterface
    {
        $criteria = $this->searchCriteriaBuilder
            ->addFilter('increment_id', $incrementId)
            ->setPageSize(1);
        $items = $this->orderRepository->getList($criteria->create())->getItems();

        if (empty($items)) {
            throw new InputException(__('Order #%1 not found in the system.', $incrementId));
        }

        return \current($items);
    }
}
